CREATE DATABASE suite
    WITH 
    OWNER = postgres
    ENCODING = 'utf8'
    TABLESPACE = pg_default
    CONNECTION LIMIT = -1;

--

CREATE SEQUENCE ID_GENERATOR;

-- version procedure
CREATE OR REPLACE FUNCTION process_version_check() RETURNS TRIGGER AS $version_check$
    BEGIN
	    IF (OLD.VERSION > NEW.VERSION) THEN 
	    	RAISE EXCEPTION 'row_version_conflict' using 
				ERRCODE = 99001,
	    		MESSAGE = ('Cannot update record % in table %. Actual version: %, provided version %', NEW.ID, TG_TABLE_NAME, OLD.VERSION, NEW.VERSION);
	    END IF;
	    
	    NEW.VERSION = OLD.VERSION + 1;
	    
        RETURN NEW;
    END;
$version_check$ LANGUAGE plpgsql;
-- end of version procedure


-- shortid procedure
CREATE OR REPLACE FUNCTION pseudo_encrypt(VALUE bigint) returns bigint AS $$
DECLARE
l1 int;
l2 int;
r1 int;
r2 int;
i int:=0;
BEGIN
 l1:= (VALUE >> 16) & 65535;
 r1:= VALUE & 65535;
 WHILE i < 3 LOOP
   l2 := r1;
   r2 := l1 # ((((1366 * r1 + 150889) % 714025) / 714025.0) * 32767)::int;
   l1 := l2;
   r1 := r2;
   i := i + 1;
 END LOOP;
 RETURN ((r1 << 16) + l1);
END;
$$ LANGUAGE plpgsql strict immutable;


CREATE OR REPLACE FUNCTION shortid(n bigint) RETURNS text
    LANGUAGE plpgsql IMMUTABLE STRICT AS $$
DECLARE
 alphabet text:='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
 base int:=length(alphabet); 
 _n bigint:=pseudo_encrypt(abs(n));
 output text:='';
BEGIN
 LOOP
   output := output || substr(alphabet, 1+(_n%base)::int, 1);
   _n := _n / base; 
   EXIT WHEN _n=0;
 END LOOP;
 RETURN output;
END $$;
-- end of shortid procedure

CREATE TABLE APP_USER (
	ID INT8 NOT NULL,
	VERSION INT8 NOT NULL DEFAULT 0,
	DELETED BOOLEAN DEFAULT FALSE,

	EMAIL VARCHAR(250) NOT NULL,
	PASSWORD VARCHAR(500),
	PASSWORD_SALT VARCHAR(250),

	FIRST_NAME VARCHAR(250),
	LAST_NAME VARCHAR(250),
	
	CONSTRAINT PK_APP_USER PRIMARY KEY(ID)
);
CREATE INDEX APP_USER_EMAIL_IDX ON APP_USER(lower(EMAIL));

CREATE TABLE APP_ROLE (
	ID INT8 NOT NULL,
	NAME VARCHAR(250) NOT NULL,
	
	CONSTRAINT PK_APP_ROLE PRIMARY KEY(ID)
);

CREATE TABLE APP_USER_ROLE (
	USER_ID INT8 NOT NULL,
	ROLE_ID INT8 NOT NULL
);
CREATE INDEX APP_USER_ROLE_IDX ON APP_USER_ROLE(USER_ID, ROLE_ID);

