package com.jh.common.security;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.context.annotation.Bean;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.SimpleUrlAuthenticationSuccessHandler;

import com.jh.common.security.repository.AppUserRepository;
import com.jh.common.security.service.AppUserAuthService;

/**
 * The Class ReactApplicationSecurityContext.
 */
public class JhSecurityConfig extends WebSecurityConfigurerAdapter {
    
    /**
     * @param http
     * @throws Exception
     * @see org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter#configure(org.springframework.security.config.annotation.web.builders.HttpSecurity)
     */
    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http
	        .authorizeRequests()
	        	.antMatchers("/auth/**").permitAll()
	        	.anyRequest().authenticated()
	            .and()
	        .logout()
	        	.clearAuthentication(true)
	            .permitAll();
        
        http.csrf().disable();
        http.exceptionHandling().authenticationEntryPoint(authenticationEntryPoint());
        http.headers().frameOptions().sameOrigin();
        
        http.sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS);
    }

    
    /**
     * @param auth
     * @throws Exception
     * @see org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter#configure(org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder)
     */
    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        auth.eraseCredentials(true);
    }    

    
    /**
     * Override this method to provide custom {@link AuthenticationEntryPoint}. 
     * 
     * It returns {@link RestAuthenticationEntryPoint} by default
     *
     * @return the authentication entry point
     */
    protected AuthenticationEntryPoint authenticationEntryPoint() {
    	return new AuthenticationEntryPoint() {
    	    @Override
    	    public final void commence(HttpServletRequest request, HttpServletResponse response, AuthenticationException authException) throws IOException {
    	        response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Unauthorized");
    	    }    		
    	};
    }

   
    protected AuthenticationSuccessHandler authenticationSuccessHandler() {
    	return new SimpleUrlAuthenticationSuccessHandler() {
    		
    		@Override
    		public void onAuthenticationSuccess( final HttpServletRequest request, final HttpServletResponse response, final Authentication authentication ) throws ServletException, IOException {
    			this.clearAuthenticationAttributes( request );
    		}
    	};
    }
    
    @Bean
    public AuthService authService() {
    	return new AppUserAuthService();
    }
    
    @Bean
    public AppUserRepository appUserRepository() {
    	return new AppUserRepository();
    }
    
}
