package com.jh.common.utils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Objects;

import org.springframework.util.StringUtils;

//import javax.mail.internet.InternetAddress;

/**
 * The Class Utils.
 */
public class Utils {

    /** The Constant ONE_HUNDRED. */
    public static final BigDecimal ONE_HUNDRED = new BigDecimal(100);

    /**
     * Parses the int.
     *
     * @param i the i
     * @param defaultValue the default value
     * @return the integer
     */
    public static Integer parseInt(String i, Integer defaultValue) {
        if (i == null) {
            return defaultValue;
        }
        try {
            return Integer.parseInt(i);
        } catch (Exception e) {
            return defaultValue;
        }
    }

    /**
     * Checks if is true.
     *
     * @param o the o
     * @return true, if is true
     */
    public static boolean isTrue(Object o) {
        if (o == null) {
            return false;
        }
        if (o instanceof Boolean) {
            return (Boolean) o;
        }

        String s = Objects.toString(o, "").toLowerCase();

        return s.equals("true") || s.equals("t") || s.equals("yes") || s.equals("y") || s.equals("1");
    }

    /* */

    /**
     * Num.
     *
     * @param a the a
     * @return the big decimal
     */
    public static BigDecimal num(BigDecimal a) {
        return a == null ? BigDecimal.ZERO : a;
    }

    /**
     * Num.
     *
     * @param a the a
     * @param fixed the fixed
     * @return the big decimal
     */
    public static BigDecimal num(BigDecimal a, int fixed) {
        return a == null ? BigDecimal.ZERO : a.setScale(fixed, RoundingMode.HALF_UP);
    }

    /**
     * Multiply.
     *
     * @param a the a
     * @param b the b
     * @return the big decimal
     */
    public static BigDecimal multiply(BigDecimal a, BigDecimal b) {
        return num(a).multiply(num(b));
    }

    /**
     * Multiply.
     *
     * @param a the a
     * @param b the b
     * @param fixed the fixed
     * @return the big decimal
     */
    public static BigDecimal multiply(BigDecimal a, BigDecimal b, int fixed) {
        return num(num(a, fixed).multiply(num(b, fixed)), fixed);
    }

    /**
     * Divide.
     *
     * @param a the a
     * @param b the b
     * @return the big decimal
     */
    public static BigDecimal divide(BigDecimal a, BigDecimal b) {
        if (b == null || BigDecimal.ZERO.compareTo(b) == 0) {
            return null;
        }
        return num(a).divide(b, RoundingMode.HALF_UP);
    }

    /**
     * Divide.
     *
     * @param a the a
     * @param b the b
     * @param fixed the fixed
     * @return the big decimal
     */
    public static BigDecimal divide(BigDecimal a, BigDecimal b, int fixed) {
        return num(num(a, fixed).divide(num(b, fixed), RoundingMode.HALF_UP), fixed);
    }

    /**
     * Plus.
     *
     * @param a the a
     * @param b the b
     * @param fixed the fixed
     * @return the big decimal
     */
    public static BigDecimal plus(BigDecimal a, BigDecimal b, int fixed) {
        return num(num(a, fixed).add(num(b, fixed)), fixed);
    }

    /**
     * Minus.
     *
     * @param a the a
     * @param b the b
     * @param fixed the fixed
     * @return the big decimal
     */
    public static BigDecimal minus(BigDecimal a, BigDecimal b, int fixed) {
        return num(num(a, fixed).subtract(num(b, fixed)), fixed);
    }

    /**
     * Plus.
     *
     * @param a the a
     * @param b the b
     * @return the big decimal
     */
    public static BigDecimal plus(BigDecimal a, BigDecimal b) {
        return num(a).add(num(b));
    }

    /**
     * Minus.
     *
     * @param a the a
     * @param b the b
     * @return the big decimal
     */
    public static BigDecimal minus(BigDecimal a, BigDecimal b) {
        return num(a).subtract(num(b));
    }

    /**
     * Compare.
     *
     * @param a the a
     * @param b the b
     * @return the int
     */
    public static int compare(BigDecimal a, BigDecimal b) {
        return num(a).compareTo(num(b));
    }

    /**
     * Sleep.
     *
     * @param millis the millis
     */
    public static void sleep(long millis) {
        try {
            Thread.sleep(millis);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * Round.
     *
     * @param n the n
     * @param fixed the fixed
     * @return the big decimal
     */
    public static BigDecimal round(BigDecimal n, int fixed) {
        if (n == null) {
            return n;
        }
        return n.setScale(fixed, RoundingMode.HALF_UP);
    }

    /**
     * Validate email.
     *
     * @param email the email
     * @return true, if successful
     */
    public static boolean validateEmail(String email) {
//        try {
//            new InternetAddress(email).validate();
//        } catch (Exception e) {
//            return false;
//        }
        return true;
    }
    
    
    public boolean hasText(String s) {
    	return StringUtils.hasText(s);
    }
    
    @SuppressWarnings("unchecked")
	public static <T> int compareValues(Comparable<T> c1, Comparable<T> c2) {
        if (c1 != null && c2 != null) {
            return c1.compareTo((T) c2);
        }
        return 0;
    }


    @SuppressWarnings("unchecked")
    public static <T> int compareValues(Comparable<T> c1, Comparable<T> c2, boolean nullsFirst) {
        if (c1 != null && c2 != null) {
            return c1.compareTo((T) c2);
        }
        if (c1 == c2) {
        	return 0;
        }
        return nullsFirst && c1 == null ? -1 : 1;
    }

    
}
