/*
 * Created on 29. 1. 2018 13:40:45
 *
 */
package com.jh.common.web;

import java.util.LinkedHashSet;
import java.util.Locale;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.i18n.AcceptHeaderLocaleResolver;
import org.springframework.web.util.WebUtils;

/**
 * The Class FallbackLocaleResolver.
 *
 * @author Jan Hrabal
 */
public class FallbackLocaleResolver extends AcceptHeaderLocaleResolver/* LocaleResolver*/ {

    /** The locales string. */
    @Value("${app.locales:en}")
    private String localesString;
    
    /** The default locale. */
    @Value("${app.locale.default:en}")
    private String defaultLocale;

    /** The default country. */
    @Value("${app.locale.defaultCountry:US}")
    private String defaultCountry;
    
    /** The locales. */
    private Set<String> locales;
    
    
    /**
     * Inits the.
     */
    @PostConstruct
    public void init() {
        locales = new LinkedHashSet<>();
        
        if (localesString != null) {
            String[] ls = localesString.split("[\\s,;]");
         
            for (String s : ls) {
                if (StringUtils.hasText(s)) {
                    locales.add(s.trim().toLowerCase());
                }
            }
        } else {
            locales.add("en");//TODO default config?
        }
        
        if (defaultLocale == null) {
            defaultLocale = "en";
        } else {
            defaultLocale = defaultLocale.trim().toLowerCase();
        }
    }

    
    /**
     * Resolve locale.
     *
     * @param httpRequest the http request
     * @return the locale
     * @see org.springframework.web.servlet.LocaleResolver#resolveLocale(javax.servlet.http.HttpServletRequest)
     */
    @Override
    public Locale resolveLocale(HttpServletRequest httpRequest) {
        String code = null;
        
        //get param
        String lang = httpRequest.getParameter("lang");
        if (StringUtils.hasText(lang)) {
            if (locales.contains(langCode(lang))) {
                code = lang;
            }
        }
        
        Cookie c = WebUtils.getCookie(httpRequest, "lang");
        if (c != null) {
            lang = c.getValue();
            if (StringUtils.hasText(lang)) {
                if (locales.contains(langCode(lang))) {
                    code = lang;
                }
            }
        }
        
        if (code == null) {
            String acceptLanguage = httpRequest.getHeader("Accept-Language");
            if (StringUtils.hasText(acceptLanguage)) {
                String[] als = acceptLanguage.split(",");
                for (String s : als) {
                    if (locales.contains(langCode(s))) {
                        code = s;
                        break;
                    }
                }
            }
        }
        
        String country = null;
        if (code == null) {
            code = defaultLocale;
            country = defaultCountry;
        } else {
            country = countryCode(code);
            code = langCode(code);
        }
        
        return country == null ? new Locale(code) : new Locale(code, country);
    }

    /**
     * Sets the locale.
     *
     * @param arg0 the arg 0
     * @param arg1 the arg 1
     * @param arg2 the arg 2
     * @see org.springframework.web.servlet.LocaleResolver#setLocale(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, java.util.Locale)
     */
    @Override
    public void setLocale(HttpServletRequest arg0, HttpServletResponse arg1, Locale arg2) {
        // TODO do nothing
    }
    
    
    /**
     * Lang code.
     *
     * @param s the s
     * @return the string
     */
    protected String langCode(String s) {
        if (s == null) {
            return defaultLocale;
        }
        s = s.trim().toLowerCase();
        if (s.length() > 2) {
            return s.substring(0, 2);
        }
        return s;
    }
    
    
    /**
     * Lang name.
     *
     * @param s the s
     * @return the string
     */
    protected String langName(String s) {
        if (s == null) {
            return defaultLocale;
        }
        return s.trim().replace("-", "_");
    }

    
    /**
     * Country code.
     *
     * @param s the s
     * @return the string
     */
    protected String countryCode(String s) {
        if (s == null || s.length() < 5 ) {
            return null;
        }
        
        return s.substring(3);
    }
}
