package com.jh.common.web.error;

import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

/**
 * The Class RestResponseEntityExceptionHandler.
 */
@ControllerAdvice
public class RestResponseEntityExceptionHandler extends ResponseEntityExceptionHandler {


	/**
	 * Handle all.
	 *
	 * @param ex the ex
	 * @return the response entity
	 */
	@ExceptionHandler({ Exception.class })
	public ResponseEntity<Object> handleAll(Exception ex) {
		ErrorMessage[] errors = null;
		
		if (ex instanceof ErrorMessages) {
			errors = ((ErrorMessages) ex).getErrorMessages();
		}
		if (errors == null) {
			errors = new ErrorMessage[] { ErrorMessage.withCodeMessage("error.internal", ex.getMessage()) };
		}
		
		HttpStatus status = null;
		if (ex instanceof ErrorStatus) {
			status = ((ErrorStatus) ex).getErrorStatus();
		}
		
		if (status == null) {
			ResponseStatus responseStatus = ex.getClass().getAnnotation(ResponseStatus.class);
			if (responseStatus != null) {
				status = responseStatus.value();
			}
		}
		
		if (status == null) {
			status = HttpStatus.INTERNAL_SERVER_ERROR;
			//log unhandled exception
			ex.printStackTrace();
		}
		
	    return new ResponseEntity<>(errors, new HttpHeaders(), status);
	}

	
}