/*
 * Created on 12. 10. 2016 13:01:20
 *
 */
package com.jh.common.web.list;

import java.util.Collections;
import java.util.List;

/**
 * TODO.
 *
 * @author Jan Hrabal
 * @param <T> the generic type
 */
public class Page<T> {

    /** The content. */
    private List<T> content;
    
    /** The pages count. */
    private int pagesCount;
    
    /** The page. */
    private int page;

    /** The sort by. */
    private String sortBy;
    
    /** The trend. */
    private SortTrend trend;
    
    /**
     * Instantiates a new page.
     *
     * @param content the content
     */
    public Page(List<T> content) {
        this(0, 1, content);
    }

    /**
     * Instantiates a new page.
     *
     * @param page the page
     * @param pagesCount the pages count
     * @param content the content
     */
    public Page(int page, int pagesCount, List<T> content) {
        this.content = content;
        this.page = page;
        this.pagesCount = pagesCount;
    }

    /**
     * Instantiates a new page.
     *
     * @param page the page
     * @param pagesCount the pages count
     * @param content the content
     * @param sortBy the sort by
     * @param trend the trend
     */
    public Page(int page, int pagesCount, List<T> content, String sortBy, SortTrend trend) {
        super();
        this.content = content;
        this.pagesCount = pagesCount;
        this.page = page;
        this.sortBy = sortBy;
        this.trend = trend;
    }

    /**
     * Gets the content.
     *
     * @return the content
     */
    public List<T> getContent() {
        return content == null ? Collections.emptyList() : content;
    }

    /**
     * Gets the pages count.
     *
     * @return the pages count
     */
    public int getPagesCount() {
        return pagesCount;
    }

    /**
     * Gets the page.
     *
     * @return the page
     */
    public int getPage() {
        return page;
    }

    /**
     * Gets the sort by.
     *
     * @return the sort by
     */
    public String getSortBy() {
        return sortBy;
    }

    /**
     * Gets the trend.
     *
     * @return the trend
     */
    public SortTrend getTrend() {
        return trend;
    }

    /**
     * Checks if is sorted.
     *
     * @return true, if is sorted
     */
    public boolean isSorted() {
        return sortBy != null && trend != null;
    }
}
