package com.jh.common.web.list;

import org.springframework.core.MethodParameter;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

import com.jh.common.utils.Utils;

/**
 * The Class SortingArgumentResolver.
 */
public class SortingArgumentResolver implements HandlerMethodArgumentResolver {
    
	/*
	const ps = headers["x-paging-page"] * 1;
	const psize = headers["x-paging-pagesize"] * 1;
	const pcs = headers["x-paging-pagescount"] * 1;	
	*/
	
    /** The Constant PAGING_PAGE. */
	private static final String PAGING_PAGE = "X-Paging-Page";
    
    /** The Constant PAGING_PAGE_SIZE. */
    private static final String PAGING_PAGE_SIZE = "X-Paging-PageSize";
//    private static final String PAGING_PAGES_COUNT = "X-Paging-PagesCount";
    

    /**
 * @param parameter
 * @return
 * @see org.springframework.web.method.support.HandlerMethodArgumentResolver#supportsParameter(org.springframework.core.MethodParameter)
 */
@Override
    public boolean supportsParameter(MethodParameter parameter) {
        return parameter.getParameterType().equals(PagingInfo.class);
    }

    
    /**
     * @param parameter
     * @param mavContainer
     * @param webRequest
     * @param binderFactory
     * @return
     * @throws Exception
     * @see org.springframework.web.method.support.HandlerMethodArgumentResolver#resolveArgument(org.springframework.core.MethodParameter, org.springframework.web.method.support.ModelAndViewContainer, org.springframework.web.context.request.NativeWebRequest, org.springframework.web.bind.support.WebDataBinderFactory)
     */
    @Override
    public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer mavContainer, NativeWebRequest webRequest, WebDataBinderFactory binderFactory) throws Exception {
    	Integer page = Utils.parseInt(webRequest.getHeader(PAGING_PAGE), 0);
    	Integer pageSize = Utils.parseInt(webRequest.getHeader(PAGING_PAGE_SIZE), 0);
        return new DefaultPagingInfo(page, pageSize);
    }
}